"""
This example shows how to use vLLM for running offline inference with
multi-image input on vision language models for text generation,
using the chat template defined by the model.
"""
from argparse import Namespace
from typing import List, NamedTuple, Optional

from PIL import Image
from transformers import AutoProcessor, AutoTokenizer

from vllm import LLM, SamplingParams
from vllm.multimodal.utils import fetch_image
from vllm.utils import FlexibleArgumentParser
import decord,cv2

import requests,base64,os
from PIL import Image
from vllm.model_executor.models.intern_vit import InternVisionEncoder
from typing import Union
from io import BytesIO

def encode_image_base64(image: Union[str, Image.Image]) -> str:
    """encode raw date to base64 format."""
    res = ''
    if isinstance(image, str):
        url_or_path = image
        if url_or_path.startswith('http'):
            FETCH_TIMEOUT = int(os.environ.get('LMDEPLOY_FETCH_TIMEOUT', 10))
            headers = {
                'User-Agent':
                'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 '
                '(KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.3'
            }
            try:
                response = requests.get(url_or_path,
                                        headers=headers,
                                        timeout=FETCH_TIMEOUT)
                response.raise_for_status()
                res = base64.b64encode(response.content).decode('utf-8')
            except Exception:
                pass
        elif os.path.exists(url_or_path):
            with open(url_or_path, 'rb') as image_file:
                res = base64.b64encode(image_file.read()).decode('utf-8')
    elif isinstance(image, Image.Image):
        buffered = BytesIO()
        image.save(buffered, format='PNG')
        res = base64.b64encode(buffered.getvalue()).decode('utf-8')
    return res

QUESTION = "What is the content of each image?"
IMAGE_URLS = [
    "https://upload.wikimedia.org/wikipedia/commons/d/da/2015_Kaczka_krzy%C5%BCowka_w_wodzie_%28samiec%29.jpg",
    "https://upload.wikimedia.org/wikipedia/commons/7/77/002_The_lion_king_Snyggve_in_the_Serengeti_National_Park_Photo_by_Giles_Laurent.jpg",
]


class ModelRequestData(NamedTuple):
    llm: LLM
    prompt: str
    stop_token_ids: Optional[List[str]]
    image_data: List[Image.Image]
    chat_template: Optional[str]


# NOTE: The default `max_num_seqs` and `max_model_len` may result in OOM on
# lower-end GPUs.
# Unless specified, these settings have been tested to work on a single L4.


def load_internvl():
    model_name = "/mnt/xnncloud/torin/workdirs/internvl_chat_v2_0/lora_3opt_v5_240910_merged"

    llm = LLM(
        # multi_step_stream_outputs = False,
        model=model_name,
        trust_remote_code=True,
        max_model_len=5096,
        limit_mm_per_prompt={"image": 16},
        mm_processor_kwargs={"max_dynamic_patch": 1},
    )
    # video_fn = '/input/projects/op_pipeline/test_high_crf.mp4' 
    # vr = decord.VideoReader(video_fn, num_threads=4)
    # imgs = vr.get_batch(range(16)).asnumpy()
    # resized_imgs = []
    # for img in imgs:
    #     resized_imgs.append(Image.fromarray(cv2.resize(img,(448,448))).convert("RGB"))
    # ↑ just give a random image with 3 448 448

    question = '"What type of animal is in the image above? What color does it have?'
    
    placeholders = "\n".join(f"Image-{i}: <image>\n"
                             for i, _ in enumerate(resized_imgs, start=1))
    messages = [{'role': 'user', 'content': f"{placeholders}\n{question}"}]

    tokenizer = AutoTokenizer.from_pretrained(model_name,
                                              trust_remote_code=True)
    prompt = tokenizer.apply_chat_template(messages,
                                           tokenize=False,
                                           add_generation_prompt=True)

    # Stop tokens for InternVL
    # models variants may have different stop tokens
    # please refer to the model card for the correct "stop words":
    # https://huggingface.co/OpenGVLab/InternVL2-2B#service
    stop_tokens = ["<|endoftext|>", "<|im_start|>", "<|im_end|>", "<|end|>"]
    stop_token_ids = [tokenizer.convert_tokens_to_ids(i) for i in stop_tokens]

    return ModelRequestData(
        llm=llm,
        prompt=prompt,
        stop_token_ids=stop_token_ids,
        image_data=resized_imgs,
        chat_template=None,
    )






def run_generate():
    req_data = load_internvl()

    sampling_params = SamplingParams(temperature=0.0,
                                     max_tokens=128,
                                     stop_token_ids=req_data.stop_token_ids)
    import time
    for _ in range(3):
      t1 = time.time()
      outputs = req_data.llm.generate(
          {
              "prompt": req_data.prompt,
              "multi_modal_data": {
                  "image": req_data.image_data
              },
          },
          sampling_params=sampling_params)

      for o in outputs:
          generated_text = o.outputs[0].text
          print(generated_text)
      print('final',time.time() - t1)

def run_chat():
    req_data = load_internvl()
    import time

    sampling_params = SamplingParams(temperature=0.0,
                                     max_tokens=128,
                                     stop_token_ids=req_data.stop_token_ids)
    question = '"What type of animal is in the image above? What color does it have?'
    for _ in range(3):
        t1 = time.time()
        outputs = req_data.llm.chat(
            [{
                "role":
                "user",
                "content": [
                    {
                        "type": "text",
                        "text": question,
                    },
                    *({
                        "type": "image_url",
                        "image_url": {
                            "url": f'data:image/png;base64,{encode_image_base64(img)}' 
                        },
                    } for img in req_data.image_data),
                ],
            }],
            sampling_params=sampling_params,
            chat_template=req_data.chat_template,
        )

        for o in outputs:
            generated_text = o.outputs[0].text
            print(generated_text)
        print('final',time.time() - t1)

if __name__ == "__main__":

    # run_generate()
    run_chat()